////////////////////////////////////////////////////////////////
// Written by M.Korn,  Nuremberg 02.08.2017 
// Fr freie Verwendung, nur wenn   Code unverndert ist.
////////////////////////////////////////////////////////////////


#include <avr/io.h>
#include <util/delay.h>
#include <stdint.h>
#include <avr/pgmspace.h>
#include <uart.c>
#include <uart.h>
#include <avr/interrupt.h>

/////////////////////////////Atmega328p  IBIS_Datenwandler  ///////////////////////////

int softdata;
int softparity;

#define  suart_port  PORTB
#define  suart_pint  PB0  //(TX,7E2,  zum IBIS)
 //  PD1 = TX
 //  PD0 = RX, Baud9600  8N1

// Prototypen
void sleep_ms (uint16_t ms);
void sleep_us (uint16_t us);
void init_timer ();

void uart_put ();

// Variable festlegen & Werte fttern
char string[26];
char parity;
int  value;
int  i;   // nur fr  UART-Daten-holen verwenden !!
int  j;
int  s;		// nur fr  Soft-UART !!
char bcd[3];
char data[48];


int main(void)
{	// initalisierte Werte

	// Port B initalisieren, PB0 als Ausgang festlegen
	DDRB= 0xFF;
	suart_port |= (1 << suart_pint);	// sTX Pin auf high setzen	DATA 0-6

	// UART initalisiern, Baudrate = 1200
	uart_init( UART_BAUD_SELECT(9600,F_CPU) ); 
	//UCSR0C = 0x2C; // UART-Datenformat auf 7E2 erzwingen. 0x2C

	// Interrupt aktivieren
	sei();	
	
    while(1) { 
	// Zyklussgeschwindigkeit
	sleep_ms (100);		// 0,5 Sekunde warten
	//  UART auslesen
	
	if ((i <=15 ) ==0 )	{		// wenn schleifenzhler  23 erreicht hat,
		uart_put ();			// Daten an UART schicken
		i= 0;					// dann auf null setzen
		}

	while(i <= 15) {  
	value = uart_getc ();		// Daten von UART holen.	
	if ( value & UART_NO_DATA )	// wenn keine Daten
        {
		break;					// dann sofort Schleife verlassen
        }
	else						//sonst Daten rausholt
		{						
		if (i == 0 ) {			// wenn Daten nun da und Schleifenzhler = 0 ist
			j= 0;				//  zuerst alte Daten lschen
			while(j <= 15) {  	// 24mal  alte Daten mit Leerzeichen berschreiben.
			data [j] =0x20 ;    
			j++;				
			}					
		}						// dann mit neue Daten fttern
		if (value == 0x0D ) {   // wenn <CR> da ist, 
			i = 0; 				// dann Schleifenzhler auf Null setzen
			uart_putc (0x0D);	// Rckmeldung
			uart_putc ('\n');	// rckmeldung
			uart_put ();		// Daten an UART schicken
			break; 				// und Schleife sofort verlassen.
			}
		if (value == 0x08){		// wenn <BS> da ist,  (Fehlerkorrektur)
			i--;					// dann Schleifenzhler um 1 zurckgehen und
			data[i] = 0x20;		// dann wird Daten mit Leerzeichen bergeschrieben.
			uart_putc (0x08);					// nochmals um 1 zurckgehen (wegen automatische inkrementierung von Schleifenzhler)
			break;
			}

		data[i] = value ;		// 24mal  Daten von UART-Puffer holen.
		uart_putc (data [i]);	// Rckmeldung	
		suart_putc (data [i]);	// Rckmeldung	
		}
	i++;
	}

    }
}

void sleep_ms (uint16_t ms)
{
	while (ms--)
        _delay_ms (1);
}

void sleep_us (uint16_t us)
{
	while (us--)
        _delay_us (1);
}


	

void uart_put ()
	{	
	//Paritt ausrechnen 
	parity = 0x7F;				//  0x09 = 0x7F ^ 'v'
	parity = parity ^ 'z';
	parity = parity ^ 'A';
	parity = parity ^ '1';
	
	j= 0;						// Paritt mit Daten 24mal ausrechnen
	while(j <= 15) {  
	parity = parity ^ data[j];	// XOR-Verknpfung von vorherige Paritt-ergebnisse
	j++;						// und Daten
	}
	parity = parity ^ 0x0D;		//
	// Daten an UART schicken
	sleep_ms(100);
	suart_putc('z'); 	// DS009 Datensatz whlen
	uart_putc('z'); 	// DS009 Datensatz whlen
	suart_putc('A'); 	// DS009 Datensatz whlen
	uart_putc('A'); 	// DS009 Datensatz whlen
	suart_putc('1'); 	// DS009 Datensatz whlen
	uart_putc('1'); 	// DS009 Datensatz whlen
		
	j= 0;				// 24mal Daten an UART schicken
	while(j <= 15) {  	
	suart_putc(data[j]);
	uart_putc(data[j]);
	j++;
	}
	suart_putc(0x0D);	// <CR> an UART schicken
	uart_putc(0x0D);	// <CR> an UART schicken
	suart_putc(parity);	// <P>  an UART schicken
	uart_putc(parity);	// <P>  an UART schicken
	suart_putc('\n');	// nicht notwendig, fr Terminalprogramm zwecks Zeilenumbruch.
	uart_putc('\n');	// nicht notwendig, fr Terminalprogramm zwecks Zeilenumbruch.
	
	}
	
#define Timing  600


void suart_putc (int softdata){				//software-UART , only Transmitting with even parity

	//gerade Paritt ausrechnen // calc even parity
	softparity = softdata &0x7F;//   0x7F Maske , 7Bit
	s= 1;						// Paritt mit Daten 7mal ausrechnen
	while(s <= 7) {  
	softparity = softparity^((softparity >>s) &0x01);	// XOR-Verknpfung von vorherige Paritt-ergebnisse
	s++;						// und Daten
	}

	
	
	// Daten schicken	// transmitting of data
	suart_port &= ~(1 << suart_pint); 	// sTX Pin auf low setzen		START
	s= 0;									// 7mal Daten an softUART schicken
	sleep_us (Timing);
	while(s <= 6) { 
	if ((softdata>>s)& 0x01)
		suart_port |= (1 << suart_pint);	// sTX Pin auf high setzen	DATA 0-6
	else
		suart_port &= ~(1 << suart_pint); 	// sTX Pin auf low setzen
	sleep_us (Timing);
	s++;
	}
	
	if (softparity & 0x01)
		suart_port |= (1 << suart_pint);	// sTX Pin auf high setzen	PARITY EVEN
	else
		suart_port &= ~(1 << suart_pint); 	// sTX Pin auf low setzen
	sleep_us (Timing);

	
	suart_port |= (1 << suart_pint);		// sTX Pin auf high setzen		STOP
	sleep_us (Timing);
	suart_port |= (1 << suart_pint);		// sTX Pin auf high setzen		STOP
	sleep_us (Timing);
	
}

